/**********************************************************************
Copyright (c) 2006 Travis Reeder and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.sql4o;

import java.util.Date;
import java.text.ParseException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 * Convert fields to different data types.
 */
public class Converter
{
    public static final String DATE_FORMAT_PATTERN = "yyyy-MM-dd";

    public static DateFormat df = new SimpleDateFormat(DATE_FORMAT_PATTERN);

    public static Boolean convertToBoolean(Object o)
    {
        if (o instanceof Boolean)
        {
            // todo: this should accept Numbers too
            return (Boolean) o;
        }
        return false;
    }

    /**
     * Converts a String value to the Class represented by to. Only converts primitives and second class types like Date
     * and String.
     * @param to
     * @param from
     * @return
     * @throws Exception
     */
    @SuppressWarnings("unchecked")
    public static Object convertFromString(Class to, String from) throws Exception
    {
        Object val = null;
        // could probably just instantiate most of these things using reflection
        if (to.isPrimitive())
        {
            if (to.isAssignableFrom(Integer.TYPE))
            {
                val = new Integer(from);
            }
            else if (to.isAssignableFrom(Long.TYPE))
            {
                val = new Long(from);
            }
            else if (to.isAssignableFrom(Float.TYPE))
            {
                val = new Float(from);
            }
            else if (to.isAssignableFrom(Double.TYPE))
            {
                val = new Double(from);
            }
            else if (to.isAssignableFrom(Short.TYPE))
            {
                val = new Short(from);
            }
            else if (to.isAssignableFrom(Byte.TYPE))
            {
                val = new Byte(from);
            }
            else if (to.isAssignableFrom(Boolean.TYPE))
            {
                val = new Boolean(from);
            }
            else if (to.isAssignableFrom(Character.TYPE))
            {
                val = parseCharacter(from);
            }

        }
        else if (Boolean.class.isAssignableFrom(to))
        {
            val = new Boolean(from);
            // todo: add the rest of the wrapper types - Boolean, Character
        }
        else if (Character.class.isAssignableFrom(to))
        {
            val = parseCharacter(from);
        }
        else if (Number.class.isAssignableFrom(to))
        {
            if (to == Integer.class)
            {
                val = new Integer(from);
            }
            else if (to == Long.class)
            {
                val = new Long(from);
            }
            else if (to == Float.class)
            {
                val = new Float(from);
            }
            else if (to == Double.class)
            {
                val = new Double(from);
            }
            else if (to == Short.class)
            {
                val = new Short(from);
            }
            else if (to == Byte.class)
            {
                val = new Byte(from);
            }
        }
        else if (to == String.class)
        {
            val = from;
        }
        else if (to == Date.class)
        {
            try
            {
                val = df.parse(from);
            }
            catch (ParseException e)
            {
                throw new Exception("Could not parse date: " + from + ". Format must be: " + DATE_FORMAT_PATTERN);
            }
        }
        else
        {
            throw new Exception("Value type is not recognized! " + to + " : " + from);
        }
        return val;
    }

    private static Object parseCharacter(String from) throws Exception
    {
        Object val;
        if (from.length() != 1)
        {
            throw new Exception("Could not parse character: " + from);
        }
        val = new Character(from.charAt(0));
        return val;
    }
}