/**********************************************************************
Copyright (c) 2006 Travis Reeder and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
 **********************************************************************/
package org.datanucleus.sql4o;

/**
 * Representation of a row of the results of an SQL query.
 */
public interface Result
{
    /**
     * Accessor for field value given the field position.
     * The getObject(String) method is recommended over this one in case the field order changes.
     * @param fieldIndex the field index within the object or the select expression
     * @return the field value for the field in the object at this index.
     */
    Object getObject(int fieldIndex) throws Sql4oException;

    /**
     * Accessor for field value given the field name.
     * Recommended over index-based access.
     * @param fieldName the name of the field in the object
     * @return the field value for the object with specified field name
     */
    Object getObject(String fieldName) throws Sql4oException;

    /**
     * @param objectIndex this is here to support joins, you will generally just use zero
     * @return the underlying object the the field values are obtained from
     */
    Object getBaseObject(int objectIndex);

    /**
     * Accessor for the number of fields in this row.
     * @return Number of fields
     */
    int getNumberOfFields();
}
