/**********************************************************************
Copyright (c) 2006 Travis Reeder and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.sql4o.metadata;

import com.db4o.ObjectSet;
import com.db4o.ObjectContainer;
import com.db4o.reflect.ReflectClass;
import com.db4o.reflect.ReflectField;
import com.db4o.reflect.generic.GenericReflector;

import java.util.List;
import java.util.ArrayList;

import org.datanucleus.sql4o.*;
import org.datanucleus.sql4o.query.From;
import org.datanucleus.sql4o.query.SqlQuery;
import org.datanucleus.sql4o.util.ReflectHelper;

/**
 * 
 */
public class ObjectSetMetaDataImpl implements ObjectSetMetaData
{
    private List<String> fields = new ArrayList<String>();

    private List<ReflectField> reflectFields = new ArrayList<ReflectField>();

    public ObjectSetMetaDataImpl(ObjectSet results, ObjectSetWrapper objectSetWrapper, ObjectContainer oc, SqlQuery sqlQuery)
    {
        init(results, objectSetWrapper, oc, sqlQuery);
    }

    private void init(ObjectSet results, ObjectSetWrapper objectSetWrapper, ObjectContainer oc, SqlQuery sqlQuery)
    {

        From from = sqlQuery.getFrom();
        List<ClassRef> classRefs = from.getClassRefs();
        for (int i = 0; i < classRefs.size(); i++)
        {
            ClassRef classRef = classRefs.get(i);
            String cname = classRef.getClassName();
            GenericReflector reflector = oc.ext().reflector();
            ReflectClass reflectClass = reflector.forName(cname);
            ReflectField reflectFields[] = getDeclaredFields(reflectClass);
            for (int j = 0; j < reflectFields.length; j++)
            {
                ReflectField reflectField = reflectFields[j];
                if (objectSetWrapper.hasSelectFields())
                {
                    if (objectSetWrapper.getSelectFields().contains(reflectField.getName()))
                    {
                        this.reflectFields.add(reflectField);
                        fields.add(reflectField.getName());
                    }
                }
                else
                {
                    this.reflectFields.add(reflectField);
                    fields.add(reflectField.getName());
                }
            }
        }

    }

    private ReflectField[] getDeclaredFields(ReflectClass reflectClass)
    {
        return ReflectHelper.getDeclaredFieldsInHeirarchy(reflectClass);
    }

    public int getColumnCount()
    {
        return fields.size();
    }

    public String getColumnName(int column)
    {
        if (column >= 0 && column < fields.size())
        {
            return fields.get(column);
        }
        return null;
    }

    public ReflectField getColumnReflectField(int fieldIndex)
    {
        if (reflectFields.size() <= fieldIndex || fieldIndex < 0)
        {
            // then out of bounds, so throw
            throw new Sql4oRuntimeException("Field index out of bounds. received: " + fieldIndex + " max: " + reflectFields.size());
        }
        else
        {
            ReflectField ret = reflectFields.get(fieldIndex);
//            ret.setAccessible();
            return ret;
        }
    }

    public ReflectField getColumnReflectField(String fieldName)
    {
        for (int i = 0; i < reflectFields.size(); i++)
        {
            ReflectField reflectField = reflectFields.get(i);
            if (reflectField.getName().equals(fieldName))
            {
//                reflectField.setAccessible();
                return reflectField;
            }
        }
        return null;
    }

}
